 /*
 *Copyright (c) 2013-2013, yinqiwen <yinqiwen@gmail.com>
 *All rights reserved.
 * 
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of Redis nor the names of its contributors may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 *BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
 *THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SOFT_SIGNAL_CHANNEL_HPP_
#define SOFT_SIGNAL_CHANNEL_HPP_
#include "channel/channel.hpp"
#include <vector>
#include <string>

namespace ardb
{
	struct EventFD
	{
		private:
			char _kReadSigInfoBuf[sizeof(uint64)];
			char _kWriteSigInfoBuf[sizeof(uint64)];
			uint32 m_readed_siginfo_len;
		public:
			int write_fd;
			int read_fd;

			EventFD() :
					m_readed_siginfo_len(0), write_fd(-1), read_fd(-1)
			{
			}
			inline bool IsOpen()
			{
				return read_fd > 0 || write_fd > 0;
			}
			int OpenNonBlock();
			int Close();
			int WriteEvent(uint64 ev);
			int ReadEvent(uint64& ev);
	};

	struct SoftSignalHandler
	{
			virtual void OnSoftSignal(uint32 soft_signo, uint32 appendinfo) = 0;
			virtual ~SoftSignalHandler()
			{
			}
	};

	class SoftSignalChannel: public Channel
	{
		private:
			typedef std::map<uint32, std::vector<SoftSignalHandler*> > SignalHandlerMap;
			SignalHandlerMap m_hander_map;

			EventFD m_event_fd;
			SoftSignalChannel(ChannelService& factory);
			void FireSignalReceived(uint32 soft_signo, uint32 append_info);
			bool DoOpen();
			bool DoClose();
			void OnRead();
			int GetWriteFD();
			int GetReadFD();
			~SoftSignalChannel();
			friend class ChannelService;
		public:
			void Register(uint32 signo, SoftSignalHandler* handler);
			void Unregister(SoftSignalHandler* handler);
			void AttachEventFD(EventFD* fd);
			int FireSoftSignal(uint32 signo, uint32 append_info);
			void Clear();
	};
}

#endif /* SOFT_SIGNAL_CHANNEL_HPP_ */
